/***************************************************************************
Copyright (c) 2006 Mateusz Bilski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_CLIENT_H
#define SMART_TOOLS_CLIENT_H

#include <netinet/in.h>
#include <netdb.h>
#include <sys/un.h>
#include <errno.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "stsocket.h"
#include "stbuffer.h"
#include "sterror.h"
#include "stassert.h"
#include "ststring.h"
#include "stthread.h"

/*!
	\brief Client of unix sockets.
	
	There are two ways of using StClient. You can set the onDataPointer on a function and start a thread.
	If new data is received, the function will be called and the received data will be added to dataBuffer.
	The another way is a loop which calls a recv function.
	
	\sa StClient
*/

class StClient : public StSocket, public StThread
{
	public:
		enum Errors { E_UNKNOWN, E_ACCESS, E_WOULD_BLOCK, E_CONNECTION_RESET, E_NOT_CONNECTED, 
			      E_HOST, E_CONNECTED, E_NETWORK, E_TIMEOUT, E_REFUSED };
			      
		StClient( const StClient& copy );
				
		/*! 	
			Create an endpoint for communication.
			\param domain the domain parameter specifies a communication domain.
		*/
		StClient( int domain = PF_UNIX ) : StSocket( domain ) { init(); lock(); }
		
		~StClient() { unlock(); clean(); }
		
		/*! 
			Connect to a server at a given address and port. 
			\warning The domain must be AF_INET or PF_INET.
		*/
		bool connect( const StString& address, int port );
		
		/*! 
			Connect to a server at a given filename. 
			\warning The domain of the socket must be AF_UNIX or PF_UNIX.
		*/
		bool connect( const StString& filename );
		
		/*! Close and recreate a socket, so it looks like a disconnection. */
		void disconnect();
		
		/*! \return true if a socket is connected. */
		bool isConnected() const { return *connected; }
		
		/*! 
			Send the server all of a buffer.
			\note Sends a header first with size of buffer.
		*/
		bool send( StBuffer buffer, int flags = 0 );
		
		/*! 
			Wait for a message from the server, receive it all and return it as an StBuffer.
			\note Receives a header with the size of the buffer and then receives the whole message.
		*/
		StBuffer recv( int flags = 0 );
		
		StString getRemoteAddr() { return *remoteAddr; };
		
		/*! 
			Set a pointer to a function which will be called when a new message is received.
			\note You must start the thread if you want to receive messages in this way.
		*/
		void setOnData( void (*func)( StClient ) ) { onDataPointer = func; };
		
		/*! 
			Clean the data buffer.
			\return received message
		*/
		StBuffer getDataBuffer();
		
		StClient& operator=( const StClient& copy );
		bool operator==( const StClient& client ) const;
	protected:
		virtual void run();
	private:
		typedef struct Header
		{
			unsigned length;
		} Header;
		
		StString* remoteAddr;
		bool* connected;
		StBuffer* dataBuffer;
		void (*onDataPointer)( StClient );
		
		unsigned* instances;
		
		void init();
		void clean();
		void lock() { /*_printf("shared (%p): instance++: ", this);*/ (*instances)++; /*_printf("%d\n", *instances);*/ }
		void unlock() { assert( *instances > 0 ); /* _printf("shared (%p): instance--: ", this); */ (*instances)--; /*_printf("%d\n", *instances);*/ }
		
		friend class StServer;
};

#endif //SMART_TOOLS_CLIENT_H
