/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_PERMISSION_H
#define SMART_TOOLS_PERMISSION_H

/*!
	\brief StPermission allows the programmer to change and set file permissions and owners.
	
	This class is useful for storing file settings like owner, group which file belongs and permissions.
*/

class StPermission
{	
	public:
		StPermission(){ init(); setPermission(6,6,6); }
		StPermission( int owner, int group, int other ){ init(); setPermission(owner,group,other); }
		StPermission( int uid, int gid ){ init(); setOwner(uid); setGroup(gid); }
		
		/*!
			Set file permissions for owner
			
			\param read - can read
			\param write - can write
			\param execute - can execute
		*/
		void setOwner( bool read, bool write, bool execute )
		{ ownerRead = read; ownerWrite = write; ownerExecute=execute; }
		
		/*!
			Set file permissions for group
			
			\param read - can read
			\param write - can write
			\param execute - can execute
		*/
		void setGroup( bool read, bool write, bool execute )
		{ groupRead = read; groupWrite = write; groupExecute=execute; }
		
		/*!
			Set file permissions for others
			
			\param read - can read
			\param write - can write
			\param execute - can execute
		*/
		void setOther( bool read, bool write, bool execute )
		{ otherRead = read; otherWrite = write; otherExecute=execute; }
		
		/*!
			Set file permissions. This function is same as chmod
			with permissions in number format e.g. chmod 522 is the
			same as setPermission(5, 2, 2)
			
			\param owner - permissions for owner
			\param group - permissions for group
			\param other - permissions for others
		*/
		void setPermission( int owner, int group, int other );
		
		/*!
			Set file owner.
			
			\param uid - User Identifier number
		*/
		void setOwner( int uid ){ this->uid = uid; }
		
		/*!
			Set group which file belong to.
			
			\param gid - Group Identifier number
		*/
		void setGroup( int gid ){ this->gid = gid; }
		
		/** Check if owner can read a file*/
		bool ownerCanRead() const { return ownerRead; }
		/** Check if owner can write to a file*/
		bool ownerCanWrite() const { return ownerWrite; }
		/** Check if owner can execute a file*/
		bool ownerCanExecute() const { return ownerExecute; }
		
		/** Check if group can read a file*/
		bool groupCanRead() const { return groupRead; }
		/** Check if group can write to a file*/
		bool groupCanWrite() const { return groupWrite; }
		/** Check if group can execute a file*/
		bool groupCanExecute() const { return groupExecute; }
		
		/** Check if others can read a file*/
		bool otherCanRead() const { return otherRead; }
		/** Check if others can write to a file*/
		bool otherCanWrite() const { return otherWrite; }
		/** Check if others can execute a file*/
		bool otherCanExecute() const { return otherExecute; }
		
		/** Check if current user can read a file*/
		bool canIRead() const;
		/** Check if current user can write to a file*/
		bool canIWrite() const;
		/** Check if current user can execute a file*/
		bool canIExecute() const;
		
		/** Set User Identifier number*/
		void setUid( int uid ){ this->uid = uid; }
		/** Set Group Identifier number*/
		void setGid( int gid ){ this->gid = gid; }
		
		/** Get User Identifier number*/
		int getUid() const { return uid; }
		/** Get Group Identifier number*/
		int getGid() const { return gid; }
	
	private:
		bool ownerRead, ownerWrite, ownerExecute;
		bool groupRead, groupWrite, groupExecute;
		bool otherRead, otherWrite, otherExecute;
		
		int uid, gid;
		
		void init(){ uid = gid = -1; ownerRead = ownerWrite = ownerExecute = groupRead = groupWrite = groupExecute = otherRead = otherWrite = otherExecute = false; }
};

#endif //SMART_TOOLS_PERMISSION_H
